#include <delays.h>

#include "my_xlcd.h"

#include "GenericTypeDefs.h"



// DATA_PORT = data;               // Write data to port
static void my_set_data_port(BYTE value)
  {
    DATA_PORT_0 = (value & 0b00000001);
    DATA_PORT_1 = (value & 0b00000010)>>1;
    DATA_PORT_2 = (value & 0b00000100)>>2;
    DATA_PORT_3 = (value & 0b00001000)>>3;
    DATA_PORT_4 = (value & 0b00010000)>>4;
    DATA_PORT_5 = (value & 0b00100000)>>5;
    DATA_PORT_6 = (value & 0b01000000)>>6;
    DATA_PORT_7 = (value & 0b10000000)>>7;    
  }

// DATA_PORT = data;               // Write data to port
static void my_set_tris_data_port(BYTE value)
  {
    TRIS_DATA_PORT_0 = (value & 0b00000001);
    TRIS_DATA_PORT_1 = (value & 0b00000010)>>1;
    TRIS_DATA_PORT_2 = (value & 0b00000100)>>2;
    TRIS_DATA_PORT_3 = (value & 0b00001000)>>3;
    TRIS_DATA_PORT_4 = (value & 0b00010000)>>4;
    TRIS_DATA_PORT_5 = (value & 0b00100000)>>5;
    TRIS_DATA_PORT_6 = (value & 0b01000000)>>6;
    TRIS_DATA_PORT_7 = (value & 0b10000000)>>7;    
  }

static void my_tris_output(void)
  {
    TRIS_DATA_PORT_0 = 0;
    TRIS_DATA_PORT_1 = 0;
    TRIS_DATA_PORT_2 = 0;
    TRIS_DATA_PORT_3 = 0;
    TRIS_DATA_PORT_4 = 0;
    TRIS_DATA_PORT_5 = 0;
    TRIS_DATA_PORT_6 = 0;
    TRIS_DATA_PORT_7 = 0;    
  }
  

static void my_tris_input(void)
  {
    TRIS_DATA_PORT_0 = 1;
    TRIS_DATA_PORT_1 = 1;
    TRIS_DATA_PORT_2 = 1;
    TRIS_DATA_PORT_3 = 1;
    TRIS_DATA_PORT_4 = 1;
    TRIS_DATA_PORT_5 = 1;
    TRIS_DATA_PORT_6 = 1;
    TRIS_DATA_PORT_7 = 1;    
  }
    
#define LCD_POWER_ON   0x1C
#define LCD_CHAR_ON    0x14
#define LCD_LINES_2    0x28
#define LCD_CONT_DARK  0x4F
#define LCD_CRLF       0xE0
   
void lcd_open(void)
  {
    TRIS_RW=0;
    TRIS_RS=0;
    TRIS_E=0;
    TRIS_RESET=0;
    
    /* Clock in the following bytes (cycle the reset line first, if you like):
      RS Pin 	Value 	Function
      Low 	0x1C 	Turn on the lcd driver power
      Low 	0x14 	Turn on the character display
      Low 	0x28 	Set two display lines (The hd66717 considers 12 characters to be a line. Our 24 character display is actually two 12-character lines right next to each other).
      Low 	0x4F 	Set to darkest contrast
      Low 	0xE0 	Set the data address to the first character
    */
   
   RESET_PIN=0;
   DELAY_LONG;   
   

   RESET_PIN=1;
   DELAY_LONG;      
   
   lcd_cmd(LCD_POWER_ON);
   DELAY_LONG;
   
   lcd_cmd(LCD_CHAR_ON);
   lcd_cmd(LCD_LINES_2);
   lcd_cmd(LCD_CONT_DARK);
   lcd_cmd(LCD_CRLF);
  }
  
void lcd_crlf(void)
  {
    lcd_cmd(LCD_CRLF);
  }
  
void lcd_printc(char c)
  {
    lcd_data(c);
  }
   
void lcd_prints(char *p)
  {
    while(*p)
      {
        lcd_data(*p);
        p++;
      }
  }
    
void lcd_reset(void)
  {
    RESET_PIN=0;
    DELAY_LONG;
    RESET_PIN=1;    
  }
    
static void lcd_byte(BYTE value, BYTE cd)
  {
    /*
      To clock in a byte:
      1. Set the E line low.
      2. Set the RS line high if you're sending data, or low if your sending a command.
      3. Set the E line high
      4. Put your data on the data lines (DB0 through DB7)
      5. Set the E line low. The E line should remain low until 
         you're ready to clock in another byte.
      If you're using a microcontroller to do this, I'd make sure 
      there was at least a microsecond between all of the above steps, 
      and at least a millisecond between clocked in bytes. 
      This is a long time to wait, but you want to be careful if you're hardwiring 
      the Read/Write line low and not reading the busy status of the LCD.
    */
    my_tris_output();
    
    RW_PIN=0;
    
    E_PIN = 0;
    
    DELAY_SHORT;
    RS_PIN = cd; /* cmd */
    
    DELAY_TAS;
    E_PIN = 1;
    
    my_set_data_port(value);
    DELAY_TAS;
    
    E_PIN = 0;
  }
  
  
  
void lcd_data(BYTE value)
  {
    lcd_byte(value,1);
  }  
  

void lcd_cmd(BYTE value)
  {
    lcd_byte(value,0);
  }    